///////////////////////////////////////////////////////////////////////////////
//
// Filename: Marriage.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  Representation of a marriage between two people.
//
// (c) Copyright R.H.Thornburrow, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree;

import com.tsac.familytree.util.*;

import java.io.PrintWriter;


public class Marriage
{
    public Person husband;
    public Person wife;
    
    public DateBase date;
    public DateBase anulled;
    
    private String _place;
    
    public Marriage( Person husband, Person wife, DateBase date )
    {
        this.husband = husband;
        this.wife = wife;
        this.date = date;
    }

    public Marriage( Person husband, Person wife, String dstr )
        throws DateException
    {
        this( husband, wife, DateUtil.toDateBase(dstr) );
    }
    
    public void setDate( String dstr )
        throws DateException
    {
        this.date = DateUtil.toDateBase(dstr);
    }
    
    public void setPlace( String place )  { _place = place; }
    
    
    public void write( PrintWriter out )
    {
        out.print( "  <marriage" );
        
        if( husband != null )
            out.print( " husband=\"" + husband.getId() + "\"" );
        
        if( wife != null )
            out.print( " wife=\"" + wife.getId() + "\"" );
        
        if( date != null )        
            out.print( " date=\"" + date + "\"" );

        if( _place != null )        
            out.print( " place=\"" + _place + "\"" );
        
        out.println( " />" );
    }
}