///////////////////////////////////////////////////////////////////////////////
//
// Filename: AncestorDiagram.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  An Ancestor Diagram.
//
// (c) Copyright R.H.Thornburrow, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.ancestor;

import com.tsac.familytree.*;
import com.tsac.familytree.gui.MainFrame;

import java.awt.*;
import java.util.ResourceBundle;

@SuppressWarnings("serial")
public class AncestorDiagram
    extends javax.swing.JComponent
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");
    private String _title = res.getString("Family_Tree");
    private Font _titleFont = new Font( "TimesRoman", Font.ITALIC, 24 );
    private Font _nameFont  = new Font( "Helvetica", Font.PLAIN, 12 );
    private Font _dateFont  = new Font( "Helvetica", Font.PLAIN, 10 );
    private FontMetrics _titleMetrics = getFontMetrics( _titleFont );
    private FontMetrics _nameMetrics  = getFontMetrics( _nameFont );
    private FontMetrics _dateMetrics  = getFontMetrics( _dateFont );

    /*package-local*/ static int namemode = 1; 

    public AncestorDiagramNode _diagroot;


    public AncestorDiagram()
    {
        this.setBackground( Color.white );
    }

    public void setRoot( Person root )
    {
        String rootName = null;
        switch(AncestorDiagram.namemode) {
        case MainFrame.NAMING_SURNAME_LAST:
        case MainFrame.NAMING_SURNAME_FIRST:
            if( !root.getSurname().equals( "" ) ) {
                switch(AncestorDiagram.namemode) {
                case MainFrame.NAMING_SURNAME_LAST:
                    rootName = ((root.getForenames() != null) ? root.getForenames() : "?")
                    + " " + root.getSurname().toUpperCase();
                    break;
                case MainFrame.NAMING_SURNAME_FIRST:
                    rootName = root.getSurname().toUpperCase() + " "
                    + ((root.getForenames() != null) ? root.getForenames() : "?");
                    break;
                }
            } else {
                rootName = root.getName();
            }
            break;
        case MainFrame.NAMING_SIMPLE_NAMES:
            rootName = root.getName();
            break;
        }

        _title = res.getString("Ancestors_of") + " " + rootName;

        _diagroot = createNode( root );
        if(_titleMetrics.stringWidth(_title) > _diagroot.getWidth())
            _diagroot.setLocation(
                    _titleMetrics.stringWidth(_title) - _diagroot.getWidth(),
                    _titleMetrics.getHeight()*2 );
        else
            _diagroot.setLocation( 0, _titleMetrics.getHeight()*2 );

        Dimension size = new Dimension( _diagroot.getWidth(), _diagroot.getY() + _diagroot.getHeight() );
        this.setMinimumSize( size );
        this.setPreferredSize( size );
    }
    
    public void setNameMode( int namemode )
    {
        AncestorDiagram.namemode = namemode;
    }
    
    private AncestorDiagramNode createNode( Person node )
    {
        AncestorDiagramNode mother = null;
        AncestorDiagramNode father = null;

        if( node.mother != null )
            mother = createNode( node.mother );

        if( node.father != null )
            father = createNode( node.father );

        if( (mother != null) || (father != null) )
            return new AncestorMarriageNode( father, mother, new AncestorPersonNode(node, this) );

        return new AncestorPersonNode( node, this );
    }

    public void paint( Graphics g, Point origin, double fac, Font nameFont, Font dateFont )
    {
        g.setColor( Color.black );
        g.setFont( _titleFont );
        g.drawString( _title, origin.x + (int)((_diagroot.getX() + _diagroot.getWidth()/2 - _titleMetrics.stringWidth(_title)/2)*fac), _titleMetrics.getHeight()/2 + _titleMetrics.getAscent() );

        if( _diagroot != null )
            _diagroot.paint( g, origin, fac, nameFont, dateFont );
    }

    public void paint( Graphics g )
    {
        this.paint( g, new Point(0,0), 1.0, _nameFont, _dateFont );
    }

    public void print( Graphics g, Point origin, Dimension size )
    {
        double xfac = (double) size.width / (double) _diagroot.getWidth();
        double yfac = (double) size.height / (double) _diagroot.getHeight();
        double fac = Math.min( xfac, yfac );

        Font nameFont = new Font( "Helvetica", Font.PLAIN, (int)(12 * fac) );
        Font dateFont = new Font( "Helvetica", Font.PLAIN, (int)(10 * fac) );

        this.paint( g, origin, fac, nameFont, dateFont );
    }
    
    /*package-local*/ FontMetrics getNameFontMetrics() {
        return _nameMetrics;
    }
    
    /*package-local*/ FontMetrics getDateFontMetrics() {
        return _dateMetrics;
    }
}