///////////////////////////////////////////////////////////////////////////////
//
// Filename: FtmlLoader.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  Family Tree Markup Language file loader.
//
// (c) Copyright R.H.Thornburrow, 2001-4
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.ftml;

import com.tsac.familytree.gui.*;
import com.tsac.familytree.util.LogBuffer;
import com.tsac.familytree.*;
import com.tsac.ftml.parser.*;

import javax.xml.parsers.*;

public class FtmlLoader
    implements FamilyTreeLoader
{
    private Marriage[] _marriages;
    private Person[] _people;
    private Person[] _rootsConventional;
    private Person _rootInverted;
    
    public Marriage[] getMarriages()  { return _marriages; }
    public Person[] getPeople()  { return _people; }
    public Person[] getConventionalRoots()  { return _rootsConventional; }
    
    public Person[] getInvertedRoots()
    {
        if( _rootInverted != null )
        {
            Person[] ret = { _rootInverted };
            return ret;
        }
        
        return null;
    }

    public boolean load( String filename )
    {
        SAXParserFactory saxpf = SAXParserFactory.newInstance();
        FtmlHandler handler = new FtmlHandler(new LogBuffer());
        
        try
        {
            SAXParser parser = saxpf.newSAXParser();
            parser.parse( new java.io.File( filename ), handler );
            System.out.println( "Parsed ok." );
        }
        catch( ParserConfigurationException e )
        {
            System.err.println( "FATAL: Unable to instantiate parser: " + e.getMessage() );
            return false;
        }
        catch( org.xml.sax.SAXException e )
        {
            System.err.println( "FATAL: SAXException: " + e.getMessage() );
            //e.printStackTrace();
            return false;
        }
        catch( java.io.IOException e )
        {
            System.err.println( "FATAL: IOException: " + e.getMessage() );
            return false;
        }

        _people = new Person[ handler.getPeople().size() ];
        handler.getPeople().copyInto( _people );
        
        for( int n = 0; n < _people.length; n++ )
            _people[n].resolve();
        
        _marriages = new Marriage[ handler.getMarriages().size() ];
        handler.getMarriages().copyInto( _marriages );

        _rootsConventional = new Person[ handler.getRoots().size() ];
        handler.getRoots().copyInto( _rootsConventional );
        
        _rootInverted     = Person.getPerson( "ThoRoH0" );
        // leave _rootInverted as null and we will generate a list on the fly
        
        System.out.println( "Loaded " + _people.length + " people and " + _marriages.length + " marriages" );
        return true;
    }
}