///////////////////////////////////////////////////////////////////////////////
//
// Filename: DocumentPrintFrame.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  The Main Window.
//
// (c) Copyright R.H.Thornburrow, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.gui;

import java.awt.*;
import javax.swing.JFrame;

import java.net.URL;

@SuppressWarnings("serial")
public class DocumentPrintFrame
    extends JFrame 
{
    public DocumentPrintFrame( URL url )
    {
        // This code is automatically generated by Visual Cafe when you add
        // components to the visual environment. It instantiates and initializes
        // the components. To modify the code, only use code syntax that matches
        // what Visual Cafe can generate, or Visual Cafe may be unable to back
        // parse your Java file into its visual environment.
        //{{INIT_CONTROLS
        getContentPane().setLayout(new BorderLayout(0,0));
        getContentPane().setBackground(java.awt.Color.white);
        setSize(405,305);
        setVisible(false);
        getContentPane().add(BorderLayout.CENTER,doc);
        doc.setBounds(0,0,405,305);
        //}}

        //{{INIT_MENUS
        //}}

        this.setPage( url );
    }

    public void setVisible(boolean b)
    {
        if (b)
            setLocation(50, 50);
        super.setVisible(b);
    }

    public void addNotify()
    {
        // Record the size of the window prior to calling parents addNotify.
        Dimension size = getSize();

        super.addNotify();

        if (frameSizeAdjusted)
            return;
        frameSizeAdjusted = true;

        // Adjust size of frame according to the insets and menu bar
        Insets insets = getInsets();
        javax.swing.JMenuBar menuBar = getRootPane().getJMenuBar();
        int menuBarHeight = 0;
        if (menuBar != null)
            menuBarHeight = menuBar.getPreferredSize().height;
        setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
    }

    // Used by addNotify
    boolean frameSizeAdjusted = false;

    //{{DECLARE_CONTROLS
    javax.swing.JEditorPane doc = new javax.swing.JEditorPane();
    //}}

    //{{DECLARE_MENUS
    //}}

    public void setPage( URL url )
    {
        // wrap jpegs in an ImageDocument
        if( url.getFile().endsWith( ".jpg" ) )
        {
            try
            {
                doc.setDocument( new ImageDocument( url.toString() ) );
            }
            catch( Exception e )
            {
                e.printStackTrace();
            }
            return;
        }

        // load normal pages as html
        try
        {
            doc.setPage( url );
        }
        catch( java.io.IOException e )
        {
            //e.printStackTrace();
            System.out.println( "IOException: " + e.getMessage() );
        }
    }

    public void doPrint()
    {
        PrintJob job = Toolkit.getDefaultToolkit().getPrintJob( this, "Timeline", null );

        if( job == null ) return;

        setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
        Graphics jg = job.getGraphics();
        Dimension psz = job.getPageDimension();

        try
        {
            Graphics pg = jg.create(
                    PageSetupDialog.leftMargin, PageSetupDialog.topMargin,
                    psz.width - (PageSetupDialog.leftMargin + PageSetupDialog.rightMargin),
                    psz.height - (PageSetupDialog.topMargin + PageSetupDialog.bottomMargin) );
            Dimension size = new Dimension(
                    psz.width - (PageSetupDialog.leftMargin + PageSetupDialog.rightMargin),
                    psz.height - (PageSetupDialog.topMargin + PageSetupDialog.bottomMargin) );

            this.setSize( size );
            this.setVisible( true );
            doc.print( pg );
            pg.dispose();
            this.setVisible( false );
        }
        catch( Exception e )
        {
            e.printStackTrace();
            System.out.println( e );
        }
        finally
        {
            jg.dispose();
            job.end();
            job = null;
            setCursor( Cursor.getDefaultCursor() );
        }
    }
}