///////////////////////////////////////////////////////////////////////////////
//
// Filename: AboutDialog.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  The About box.
//
// (c) Copyright R.H.Thornburrow, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.gui;

//import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import com.tsac.familytree.util.LogBuffer;


@SuppressWarnings("serial")
public class ErrorsAndWarningsDialog
    extends java.awt.Dialog
{
    private JScrollPane logScl;
    private javax.swing.JTextArea logTxt;
    private javax.swing.JButton detailsBtn;
    
    public ErrorsAndWarningsDialog( LogBuffer log, java.awt.Frame parent, boolean modal )
    {
        super(parent, modal);

        try {
            setLayout(new GridBagLayout());
            //setBackground(new java.awt.Color(204,204,204));
            setSize(500, 100);
            setVisible(false);
            javax.swing.JLabel iconLbl = new javax.swing.JLabel();
            iconLbl.setIcon( new ImageIcon(ClassLoader.getSystemResource(
                    log.hasErrors() ? "images/error.gif" : "images/warning.gif")) );
            add(iconLbl, new FamilyTreeGridBagConstraints(
                    0,0,1,1,0.0,0.0, java.awt.GridBagConstraints.CENTER,
                    java.awt.GridBagConstraints.NONE, new Insets(8,8,4,4),0,0));
            javax.swing.JLabel messageLbl = new javax.swing.JLabel();
            messageLbl.setText("There were problems loading the file");
            add(messageLbl, new FamilyTreeGridBagConstraints(
                    1,0,1,1,1.0,0.0, java.awt.GridBagConstraints.CENTER,
                    java.awt.GridBagConstraints.HORIZONTAL, new Insets(8,8,4,4),0,0));
            //messageLbl.setFont(new Font("Dialog", Font.BOLD, 14));
            messageLbl.setBounds(120,12,216,24);
            
            JPanel btnPanel = new JPanel();
            btnPanel.setLayout(new GridLayout(1, 2, 10, 0));
            add(btnPanel, new FamilyTreeGridBagConstraints(
                    0,1,2,1,1.0,0.0, java.awt.GridBagConstraints.EAST,
                    java.awt.GridBagConstraints.NONE, new Insets(8,8,4,4),0,0));
            
            javax.swing.JButton okBtn = new javax.swing.JButton();
            okBtn.setText("OK");
            okBtn.setActionCommand("OK");
            btnPanel.add(okBtn);

            detailsBtn = new javax.swing.JButton();
            detailsBtn.setText("Details >>>");
            //detailsBtn.setActionCommand("OK");
            btnPanel.add(detailsBtn);


            logScl = new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            add(logScl, new FamilyTreeGridBagConstraints(
                    0,2,2,1,1.0,1.0, java.awt.GridBagConstraints.CENTER,
                    java.awt.GridBagConstraints.BOTH, new Insets(8,8,4,4),0,0));

            logTxt = new JTextArea(log.getMessages(), 15, 80);
            logTxt.setEditable(false);
            logScl.getViewport().add(logTxt);
            logScl.setVisible(false);

            setTitle("FamilyTree - Errors and Warnings");

            SymWindow aSymWindow = new SymWindow();
            this.addWindowListener(aSymWindow);
            okBtn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    okBtn_actionPerformed(event);
                }            
            });
            detailsBtn.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    detailsBtn_actionPerformed(event);
                }            
            });
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    public ErrorsAndWarningsDialog( LogBuffer log, java.awt.Frame parent, String title, boolean modal)
    {
        this(log, parent, modal);
        setTitle(title);
    }

    public void addNotify()
    {
        // Record the size of the window prior to calling parents addNotify.
        java.awt.Dimension d = getSize();

        super.addNotify();

        // Only do this once.
        if (fComponentsAdjusted)
            return;

        // Adjust components according to the insets
        java.awt.Insets insets = getInsets();
        setSize(insets.left + insets.right + d.width, insets.top + insets.bottom + d.height);
        java.awt.Component components[] = getComponents();
        for (int i = 0; i < components.length; i++)
        {
            java.awt.Point p = components[i].getLocation();
            p.translate(insets.left, insets.top);
            components[i].setLocation(p);
        }

        // Used for addNotify check.
        fComponentsAdjusted = true;
    }

    public void setVisible(boolean b)
    {
        if (b)
        {
            java.awt.Rectangle bounds = getParent().getBounds();
            java.awt.Rectangle abounds = getBounds();

            setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
                    bounds.y + (bounds.height - abounds.height)/2);
        }

        super.setVisible(b);
    }


    // Used for addNotify check.
    boolean fComponentsAdjusted = false;

    class SymWindow extends java.awt.event.WindowAdapter
    {
        public void windowClosing(java.awt.event.WindowEvent event)
        {
            Object object = event.getSource();
            if (object == ErrorsAndWarningsDialog.this)
                AboutDialog_WindowClosing(event);
        }
    }

    void AboutDialog_WindowClosing(java.awt.event.WindowEvent event)
    {
        try {
            this.dispose();
        } catch (Exception e) {
        }
    }


    void okBtn_actionPerformed(ActionEvent event)
    {
        try {
            // AboutDialog Hide the AboutDialog
            this.setVisible(false);
        } catch (java.lang.Exception e) {
        }
    }


    void detailsBtn_actionPerformed(ActionEvent event)
    {
        try {
            if(logScl.isVisible()) {
                detailsBtn.setText("Details >>>");
                logScl.setVisible(false);
                setSize(500, 130);
                this.validate();
            } else {
                detailsBtn.setText("Details <<<");
                logScl.setVisible(true);
                setSize(500, 400);
                //logScl.doLayout();
                this.validate();
            }
        } catch (java.lang.Exception e) {
        }
    }
}
