///////////////////////////////////////////////////////////////////////////////
//
// Filename: FamilyTreeTableModel.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  Model for the listview.
//
// (c) Copyright R.H.Thornburrow, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.gui;

import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.table.AbstractTableModel;

@SuppressWarnings("serial")
public class FamilyTreeTableModel
    extends AbstractTableModel
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");
    
    private String[] columnNames = {
        res.getString("Surname"), res.getString("Forenames"),
        res.getString("Relationship"), res.getString("Born"),
        res.getString("Died"), res.getString("Age") };
    private Vector<FamilyTreeTableRowModel> rows = new Vector<FamilyTreeTableRowModel>();

    public int getColumnCount()  { return columnNames.length; }

    @SuppressWarnings("unchecked")
    public Class getColumnClass( int col )  { return String.class; }

    public String getColumnName( int col )  { return columnNames[ col ]; }

    public int getRowCount()  { return rows.size(); }

    public void addRow( FamilyTreeTableRowModel row )
    {
        rows.addElement( row );
        this.fireTableRowsInserted( rows.size() - 1, rows.size() );
    }

    public FamilyTreeTableRowModel getRow( int index )
    {
        if( (index < 0) || (index >= rows.size()) )
            return null;

        return (FamilyTreeTableRowModel) rows.elementAt( index );
    }

    public void removeAllRows()  { rows.removeAllElements(); }

    public Object getValueAt( int rowIndex, int columnIndex )
    {
        if( (rowIndex < 0) || (rowIndex >= rows.size())
            || (columnIndex < 0) || (columnIndex > 5) )
            return "";

        FamilyTreeTableRowModel row = (FamilyTreeTableRowModel) rows.elementAt( rowIndex );

        switch( columnIndex )
        {
        case 0: return row.getSurname();
        case 1: return row.getForenames();
        case 2: return row.getRelationship();
        case 3: return row.getBorn();
        case 4: return row.getDied();
        case 5: return row.getAge();
        }

        return "";
    }

}