///////////////////////////////////////////////////////////////////////////////

//Filename: MainFrame.java

//Author:   Robert Thornburrow
//Project:  FamilyTree
//Created:  20th May 2001

//Purpose:  The Main Window.

//(c) Copyright R.H.Thornburrow, 2001-4

//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

///////////////////////////////////////////////////////////////////////////////

//Contributors: Janne Ilonen <janne.ilonen@virpi.net> Added *.properties files
//for multi language supports. Used JBuilder5 Wizards - Resource Strings...
//Date 2002-09-04
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.gui;

import com.tsac.familytree.*;
import com.tsac.familytree.csv.CsvLoader;
import com.tsac.familytree.gedcom.GedcomLoader;
import com.tsac.familytree.diagram.FTDiagramFrame;
import com.tsac.familytree.ancestor.AncestorDiagramFrame;
import com.tsac.familytree.timeline.TLDiagramFrame;
import com.tsac.familytree.util.LogBuffer;
import com.tsac.familytree.xml.GenericXmlLoader;

import java.awt.BorderLayout;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Toolkit;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.html.HTMLDocument;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.util.Vector;
import java.io.StringWriter;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ResourceBundle;


@SuppressWarnings("serial")
public class MainFrame
    extends javax.swing.JFrame
    implements Runnable
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");
    public static String version = "1.1.3";

    private FamilyTreeModel _model;
    private FamilyTreeTableModel _tableModel = new FamilyTreeTableModel();
    //private Person _root;
    private int[] _prunedRootsCon;
    private int[] _prunedRootsInv;
    private Person[] _people;

    private String _filename = "treedata.xml";

    private FamilyTreeNode _lastSelectedNode = null;
    private int _showFlags   = SHOW_ALL;
    private int _namemode    = NAMING_SURNAME_LAST;
    private int _orientation = FamilyTreeModel.INVERTED;
    private int _selectedConRoot = 0;
    private int _selectedInvRoot = 0;
    private boolean _prune1 = false;
    private boolean _prune2 = false;

    private Vector<URL> _history = new Vector<URL>();

    public static final int SHOW_PARENTS     = 0x01;
    public static final int SHOW_SPOUSES     = 0x02;
    public static final int SHOW_CHILDREN    = 0x04;
    public static final int SHOW_SIBLINGS    = 0x08;
    public static final int SHOW_AUNTSUNCLES = 0x10;
    public static final int SHOW_COUSINS     = 0x20;
    public static final int SHOW_ALL         = 0xff;
    
    public static final int NAMING_SURNAME_LAST  = 1;
    public static final int NAMING_SURNAME_FIRST = 2;
    public static final int NAMING_SIMPLE_NAMES  = 3;

    public abstract class MyAction
        extends AbstractAction
    {
        public MyAction(String lblkey, String icon, char accel) {
            super(res.getString(lblkey),
                    new ImageIcon(ClassLoader.getSystemResource("images/" + icon)));
            putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(accel,
                    Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
            putValue(SHORT_DESCRIPTION, res.getString(lblkey + "_Desc"));
        }
    }
    
    public MainFrame()
    {
        tree.setModel( new FamilyTreeModel() );

        // This code is automatically generated by Visual Cafe when you add
        // components to the visual environment. It instantiates and initializes
        // the components. To modify the code, only use code syntax that matches
        // what Visual Cafe can generate, or Visual Cafe may be unable to back
        // parse your Java file into its visual environment.
        //{{##INIT_CONTROLS
        setJMenuBar(menubar);
        getContentPane().setLayout(new BorderLayout());
        setSize(800,600);
        setVisible(false);
        getContentPane().add(toolbar, BorderLayout.NORTH);
        getContentPane().add(horizsplit, BorderLayout.CENTER);
        horizsplit.setBounds(0,0,625,414);
        treescrollpane.setAutoscrolls(true);
        treescrollpane.setOpaque(true);
        horizsplit.add(treescrollpane,JSplitPane.LEFT);
        treescrollpane.setBounds(1,1,250,431);
        tree.setShowsRootHandles(true);
        tree.setRowHeight(16);
        tree.setAutoscrolls(true);
        treescrollpane.getViewport().add(tree);
        tree.setBounds(0,0,247,428);
        vertsplit.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);
        horizsplit.add(vertsplit,JSplitPane.RIGHT);
        vertsplit.setBounds(0,0,375,414);
        tablescrollpane.setOpaque(true);
        vertsplit.add(tablescrollpane,JSplitPane.TOP);
        tablescrollpane.setBounds(0,0,375,175);
        table.setShowHorizontalLines(false);
        tablescrollpane.getViewport().add(table);
        table.setBounds(0,0,375,175);
        docscrollpane.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        docscrollpane.setOpaque(true);
        vertsplit.add(docscrollpane,JSplitPane.BOTTOM);
        docscrollpane.setBounds(0,0,20,40);
        doc.setEditable(false);
        docscrollpane.getViewport().add(doc);
        doc.setBounds(0,0,448,37);
        statusTxt.setEditable(false);
        getContentPane().add(statusTxt, BorderLayout.SOUTH);
        statusTxt.setBounds(0,414,625,19);
        //$$ menubar.move(48,434);
        fileMenu.setText(res.getString("File"));
        menubar.add(fileMenu);
        fileOpenBtn.setText(res.getString("Open_"));
        fileMenu.add(fileOpenBtn);
        filePageSetupBtn.setText(res.getString("Page_Setup_"));
        fileMenu.add(filePageSetupBtn);
        fileExitBtn.setText(res.getString("Exit"));
        fileMenu.add(fileExitBtn);
        viewMenu.setText(res.getString("View"));
        menubar.add(viewMenu);
        showMenu.setText(res.getString("Show"));
        viewMenu.add(showMenu);
        showParentsChk.setText(res.getString("Parents"));
        showMenu.add(showParentsChk);
        showChildrenChk.setText(res.getString("Children"));
        showMenu.add(showChildrenChk);
        showSiblingsChk.setText(res.getString("Siblings"));
        showMenu.add(showSiblingsChk);
        showAuntsUnclesChk.setText(res.getString("Aunts_Uncles"));
        showMenu.add(showAuntsUnclesChk);
        showSpousesChk.setText(res.getString("Spouses"));
        showMenu.add(showSpousesChk);
        showCousinsChk.setText(res.getString("Cousins"));
        showMenu.add(showCousinsChk);
        namingMenu.setText(res.getString("Naming"));
        viewMenu.add(namingMenu);
        namingSurnameLastRb.setText(res.getString("Surname_Last"));
        namingMenu.add(namingSurnameLastRb);
        namingSurnameFirstRb.setText(res.getString("Surname_First"));
        namingMenu.add(namingSurnameFirstRb);
        namingSimpleNamesRb.setText(res.getString("Simple_Names"));
        namingMenu.add(namingSimpleNamesRb);
        orientMenu.setText(res.getString("Orientation"));
        viewMenu.add(orientMenu);
        orientNormalRb.setText(res.getString("Normal"));
        orientMenu.add(orientNormalRb);
        orientInvertedRb.setText(res.getString("Inverted"));
        orientMenu.add(orientInvertedRb);
        viewSetRootMenu.setText(res.getString("Set_Root"));
        viewMenu.add(viewSetRootMenu);
        toolsMenu.setText(res.getString("Tools"));
        menubar.add(toolsMenu);
        // new in 1.1.3
        toolsOptionsMenu.setText(res.getString("Options"));
        toolsMenu.add(toolsOptionsMenu);
        toolsOptionsPrune1Chk.setText(res.getString("Prune1"));
        toolsOptionsMenu.add(toolsOptionsPrune1Chk);
        toolsOptionsPrune2Chk.setText(res.getString("Prune2"));
        toolsOptionsMenu.add(toolsOptionsPrune2Chk);
        //
        toolsMenu.add(drawTreeAction);
        toolsMenu.add(drawAncestorsAction);
        toolsMenu.add(drawTimelineAction);
        helpMenu.setText(res.getString("Help"));
        menubar.add(helpMenu);
        helpAboutBtn.setText(res.getString("About_"));
        helpMenu.add(helpAboutBtn);
        //$$ JMenuBar1.move(48,434);
        //##}}

        //{{INIT_MENUS
        //}}

        //{{REGISTER_LISTENERS
        SymAction lSymAction = new SymAction();
        fileExitBtn.addActionListener(lSymAction);
        //}}

        // Toolbar
        toolbar.add(drawTreeAction);
        toolbar.add(drawAncestorsAction);
        toolbar.add(drawTimelineAction);

        // Build doc popup menu
        _docPopupMenu.add( _docPopupBackBtn );
        _docPopupMenu.add( _docPopupPrintBtn );

        this.setTitle( "Taurus FamilyTree" );
        table.setModel(_tableModel);

        statusTxt.setBackground( Color.lightGray );
        tablescrollpane.setBackground( Color.white );
        tree.setCellRenderer( new FamilyTreeNodeRenderer() );
        tree.setRootVisible( false ); /*RHT*/

        // Add a listener to the tree to allows list and doc updates
        tree.addTreeSelectionListener( new TreeSelectionListener() {
            public void valueChanged( TreeSelectionEvent e ) {
                Object sel = e.getPath().getLastPathComponent();
                if( sel != null ) actionTreeNodeSelected( (FamilyTreeNode) sel );
            }
        });

        // add a listener to give tree a popup menu
        tree.addMouseListener( new MouseAdapter() {
            public void mouseClicked( MouseEvent e ) {
                if( (e.getModifiers() & InputEvent.BUTTON3_MASK) != 0 )
                    actionTreePopup( e.getX(), e.getY() );
            }
        } );

        // Add a listener to the list to allow doc updates
        table.getSelectionModel().addListSelectionListener( new ListSelectionListener() {
            public void valueChanged( ListSelectionEvent e ) {
                if( !e.getValueIsAdjusting() )
                    actionListRowSelected( table.getSelectedRow() );
            }
        });

        // Add a listener to make our hyperlinks do something
        doc.addHyperlinkListener( new HyperlinkListener() {
            public void hyperlinkUpdate( HyperlinkEvent e ) {
                if( e.getEventType() == HyperlinkEvent.EventType.ACTIVATED ) {
                    actionHyperlinkActivated( e.getURL() );
                } else if( e.getEventType() == HyperlinkEvent.EventType.ENTERED ) {
                    statusTxt.setText( (e.getURL() != null) ? e.getURL().toString() : "null" );
                } else if( e.getEventType() == HyperlinkEvent.EventType.EXITED ) {
                    statusTxt.setText( "" );
                }
            }
        } );

        // add a listener to give doc a popup menu
        doc.addMouseListener( new MouseAdapter() {
            public void mouseClicked( MouseEvent e ) {
                if( (e.getModifiers() & InputEvent.BUTTON3_MASK) != 0 )
                    actionDocumentPopup( e.getX(), e.getY() );
            }
        } );

        // Add window close listener
        this.addWindowListener( new WindowAdapter() {
            public void windowClosing( WindowEvent e ) {
                System.exit(0);
            }
        } );

        // Setup the scrollbars for the tree
        treescrollpane.setMinimumSize( tree.getPreferredScrollableViewportSize() );
        treescrollpane.setMaximumSize( new Dimension( 65536, 65536 ) );

        // Move the splitter bars
        horizsplit.setDividerLocation( 0.33 );
        vertsplit.setDividerLocation( 0.4 );

        // Menu handling code
        fileOpenBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionOpen();
            }
        });

        filePageSetupBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionPageSetup();
            }
        });

        ActionListener showAction = new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionShowChanged();
            }
        };
        showParentsChk.addActionListener( showAction );
        showSpousesChk.addActionListener( showAction );
        showChildrenChk.addActionListener( showAction );
        showSiblingsChk.addActionListener( showAction );
        showAuntsUnclesChk.addActionListener( showAction );
        showCousinsChk.addActionListener( showAction );

        ActionListener namingAction = new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionNamingChanged();
            }
        };
        namingSurnameLastRb.addActionListener( namingAction );
        namingSurnameFirstRb.addActionListener( namingAction );
        namingSimpleNamesRb.addActionListener( namingAction );

        ActionListener orientAction = new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionOrientChanged();
            }
        };
        orientNormalRb.addActionListener( orientAction );
        orientInvertedRb.addActionListener( orientAction );

        toolsOptionsPrune1Chk.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionOptionsPrune1();
            }
        } );

        toolsOptionsPrune2Chk.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionOptionsPrune2();
            }
        } );

        helpAboutBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                new AboutDialog( MainFrame.this, true ).setVisible(true); //.show();
            }
        } );

        ButtonGroup namingGrp = new ButtonGroup();
        namingGrp.add( namingSurnameLastRb );
        namingGrp.add( namingSurnameFirstRb );
        namingGrp.add( namingSimpleNamesRb );

        ButtonGroup orientGrp = new ButtonGroup();
        orientGrp.add( orientNormalRb );
        orientGrp.add( orientInvertedRb );

        _orientation = FamilyTreeModel.INVERTED;
        buildSetRootMenu();

        updateScreenState();

        // Handle history back menu button
        _docPopupBackBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionHistoryBack();
            }
        } );

        // Handle document print menu button
        _docPopupPrintBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent event ) {
                actionDocumentPrint();
            }
        } );
    }


    public void run()
    {
        //String extn = "";
        LogBuffer log = new LogBuffer();
        FamilyTreeLoader loader;

        String lower = _filename.toLowerCase();

        if( lower.endsWith( ".csv" ) )
            loader = new CsvLoader(log);
        else if( lower.endsWith( ".ged" ) )
            loader = new GedcomLoader(log);
        else
            loader = new GenericXmlLoader(log);

        //_root = null;
        _prunedRootsCon = null;
        _prunedRootsInv = null;
        _people = null;
        Person.clearKeys();
        _lastSelectedNode = null;
        tree.setModel( new FamilyTreeModel() );

        setBusy( res.getString("Loading") + " " + _filename + "'..." );

        long start = System.currentTimeMillis();
        boolean ok = loader.load( _filename );
        long end = System.currentTimeMillis();

        System.out.println( res.getString("Loaded") + " " + _filename + " " + res.getString("in") + " " + (end - start) + " ms" );
        setBusy( null );

        if( ok ) {
            populateModel( loader );
            tree.setModel( _model );
        }

        buildSetRootMenu();
        updateScreenState();

        FamilyTreeNode root = (FamilyTreeNode) tree.getModel().getRoot();
        if( root != null )
            actionTreeNodeSelected( root );
        
        if(log.hasErrors() || log.hasWarnings()) {
            new ErrorsAndWarningsDialog(log, this, false)
                .setVisible(true);
        }
    }


    public void setFilename( String filename )  { _filename = filename; }


    /**
     * Take the data from the loader and perform various calculations on it.
     * 1) Get the possible roots (normal and inverted)
     * 2) Calculate siblings (people with same father or mother)
     * 3) Make spouses aware of one another (i.e. bypass marriage object)
     * 4) If loader didn't supply roots, calculate them
     * 5) Instantiate a new FamilyTreeModel to drive tree- and table-views.
     */
    public void populateModel( FamilyTreeLoader loader  )
    {
        Marriage[] marriages = loader.getMarriages();
        _people = loader.getPeople();
        //_rootsInverted = loader.getInvertedRoots();
        //_rootsConventional = loader.getConventionalRoots();
        _prunedRootsCon = null;
        _prunedRootsInv = null;

        // Calculate children and siblings
        for( Person child : _people ) {
            // This node is a sibling of those with the same father and/or mother
            for( Person sibling : _people ) {
                if( child != sibling ) {
                    if( (child.father != null) && (sibling.father == child.father ) )
                        child.addSibling( sibling );
                    else if( (child.mother != null) && (sibling.mother == child.mother ) )
                        child.addSibling( sibling );
                }
            }
        }

        // Add spouses from marriage table
        for( Marriage marr : marriages ) {
            if( (marr.wife != null) && (marr.husband != null) ) {
                marr.wife.addSpouse( marr.husband );
                marr.husband.addSpouse( marr.wife );
            }
        }

        // set selected roots
        _selectedConRoot = 0;
        _selectedInvRoot = 0;

        // Filter the roots based on the default pruning options
        _prune1 = true;
        _prune2 = false;
        filterRoots();

        //_root = (_orientation == FamilyTreeModel.CONVENTIONAL)
        //        ? _people[_selectedConRoot] : _people[_selectedInvRoot];

        // create a new model
        //_model = new FamilyTreeModel( _people, _root, _orientation );
        _model = new FamilyTreeModel( _people, (_orientation == FamilyTreeModel.CONVENTIONAL)
                ? _prunedRootsCon : _prunedRootsInv, _orientation );
    }
    
    
    /**
     * Display the given person in the table view
     */
    private void displayPersonInTable(Person person)
    {
        setBusy( res.getString("Calculating_") );

        try {
            _tableModel.removeAllRows();

            _tableModel.addRow( new FamilyTreeTableRowModel( person, " ", " ", " " ) );

            // father
            if( isShowing(SHOW_PARENTS) && (person.father != null) )
                _tableModel.addRow( new FamilyTreeTableRowModel( person.father, res.getString("father"), res.getString("ERROR_"), res.getString("father") ) );

            // mother
            if( isShowing(SHOW_PARENTS) && (person.mother != null) )
                _tableModel.addRow( new FamilyTreeTableRowModel( person.mother, res.getString("ERROR_"), res.getString("mother"), res.getString("mother") ) );

            // husbands or wives
            if( isShowing(SHOW_SPOUSES) && (person.spouses != null) ) {
                for( Person spouse : person.spouses )
                    _tableModel.addRow( new FamilyTreeTableRowModel( spouse, res.getString("husband"), res.getString("wife"), res.getString("spouse") ) );
            }

            // sons and daughters
            if( isShowing(SHOW_CHILDREN) && (person.children != null) ) {
                for( Person child : person.children )
                    _tableModel.addRow( new FamilyTreeTableRowModel( child, res.getString("son"), res.getString("daughter"), res.getString("child") ) );
            }

            // brothers and sisters
            if( isShowing(SHOW_SIBLINGS) && (person.siblings != null) ) {
                for( Person sibling : person.siblings ) {
                    if( (person.father != sibling.father)
                            || (person.mother != sibling.mother) )
                        _tableModel.addRow( new FamilyTreeTableRowModel( sibling, res.getString("half_brother"), res.getString("half_sister"), res.getString("half_sibling") ) );
                    else
                        _tableModel.addRow( new FamilyTreeTableRowModel( sibling, res.getString("brother"), res.getString("sister"), res.getString("sibling") ) );
                }
            }

            // aunts and uncles on fathers-side
            if( isShowing(SHOW_AUNTSUNCLES) && (person.father != null) && (person.father.siblings != null) ) {
                for( Person sibling : person.father.siblings )
                    _tableModel.addRow( new FamilyTreeTableRowModel( sibling, res.getString("uncle_father_"), res.getString("aunt_father_"), res.getString("uncle_aunt_father_") ) );
            }

            // aunts and uncles on mothers-side
            if( isShowing(SHOW_AUNTSUNCLES) && (person.mother != null) && (person.mother.siblings != null) ) {
                for( Person sibling : person.mother.siblings )
                    _tableModel.addRow( new FamilyTreeTableRowModel( sibling, res.getString("uncle_mother_"), res.getString("aunt_mother_"), res.getString("uncle_aunt_mother_") ) );
            }

            // cousins on fathers-side
            if( isShowing(SHOW_COUSINS) && (person.father != null) && (person.father.siblings != null) ) {
                for( Person auntOrUncle : person.father.siblings ) {
                    if( auntOrUncle.children != null ) {
                        for( Person cousin : auntOrUncle.children )
                            _tableModel.addRow( new FamilyTreeTableRowModel( cousin, res.getString("cousin_father_"), res.getString("cousin_father_"), res.getString("cousin_father_") ) );
                    }
                }
            }

            // cousins on mothers-side
            if( isShowing(SHOW_COUSINS) && (person.mother != null) && (person.mother.siblings != null) ) {
                for( Person auntOrUncle : person.mother.siblings ) {
                    if( auntOrUncle.children != null ) {
                        for( Person cousin : auntOrUncle.children )
                            _tableModel.addRow( new FamilyTreeTableRowModel( cousin, res.getString("cousin_mother_"), res.getString("cousin_mother_"), res.getString("cousin_mother_") ) );
                    }
                }
            }
        } catch( Exception e ) {
            reportException( e );
        }

        try {
            if( person.doc != null ) {
                actionDisplayDocument( pageToURL(person.doc), true, true );
            } else {
                _history.removeAllElements();
                //doc.setDocument( new javax.swing.text.html.HTMLDocument() );
                createPageForPerson(person);
            }
        } catch( Exception e ) {
            reportException( e );
        }

        setBusy( null );        
    }

    /**
     * User has clicked on a node in the tree.  Calcuate the relationships of
     * all close relatives: Parents, Spouses, Children, Siblings, Aunts, Uncles
     * and Cousins.
     */
    private void actionTreeNodeSelected( FamilyTreeNode node )
    {
        _lastSelectedNode = node;
        displayPersonInTable(node.person);
    }


    /**
     * User has selected a row in the table-view.  Display the corresponding
     * document in the lower pane.
     */
    private void actionListRowSelected( int index )
    {
        FamilyTreeTableRowModel row = _tableModel.getRow( index );
        if( row == null )
            return;

        try {
            if( row.person.doc != null ) {
                actionDisplayDocument( pageToURL(row.person.doc), true, true );
            } else {
                _history.removeAllElements();
                //doc.setDocument( new javax.swing.text.html.HTMLDocument() );
                createPageForPerson(row.person);
            }
        } catch( Exception e ) {
            reportException( e );
        }

        setBusy( null );
    }


    public void actionHyperlinkActivated( URL url )
    {
        actionDisplayDocument( url, false, true );
    }


    public void actionHistoryBack()
    {
        if( _history.size() > 1 ) {
            _history.removeElementAt( _history.size() -1 );
            URL url = (URL) _history.lastElement();

            actionDisplayDocument( url, false, false );
        }
    }


    private URL pageToURL( String page )
    {
        if( page.indexOf( '.' ) == -1 )
            page += ".html";

        page = "html/" + page;

        return ClassLoader.getSystemResource( page );
    }


    private void actionDisplayDocument( URL url, boolean clearHistory, boolean updateHistory )
    {
        if( url == null ) {
            System.out.println( res.getString("WARNING_Attempt_to") );
            return;
        }

        // TODO: Change "ftp" to something of our own
        if( "ftp".equals(url.getProtocol()) ) {
            boolean found = false;
            String id = url.getFile();
            System.out.println(id);
            for(Person person : _people ) {
                if(id.equals(person.getId())) {
                    if(person.doc != null) {
                        try {
                            url = new URL(doc.getPage(), person.doc + ".html");
                            displayPersonInTable(person);
                            found = true;
                        } catch(MalformedURLException e) {
                            e.printStackTrace();
                        }
                    }
                    break;
                }
            }
            if(!found)
                return;
        }

        // wrap jpegs in an ImageDocument
        if( (url.getFile() != null) && url.getFile().endsWith( ".jpg" ) ) {
            try {
                doc.setDocument( new ImageDocument( url.toString() ) );
                docscrollpane.setHorizontalScrollBarPolicy( javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED );

                if( clearHistory )
                    _history.removeAllElements();

                if( updateHistory )
                    _history.addElement( url );
            } catch( Exception e ) {
                e.printStackTrace();
            }
            return;
        }

        // load normal pages as html
        try {
            setBusy( res.getString("Loading") + url + res.getString("KEY") );
            doc.setDocument( new HTMLDocument() );
            doc.setPage( url );
            docscrollpane.setHorizontalScrollBarPolicy( javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER );

            if( clearHistory )
                _history.removeAllElements();

            if( updateHistory )
                _history.addElement( url );
        } catch( java.io.IOException e ) {
            //e.printStackTrace();
            System.out.println( res.getString("IOException_") + e.getMessage() );
        }

        setBusy( null );
    }


    public void actionDocumentPrint()
    {
        DocumentPrintFrame dpf = new DocumentPrintFrame( (URL) _history.lastElement() );
        dpf.doPrint();
    }


    public void actionDocumentPopup( int x, int y )
    {
        _docPopupBackBtn.setEnabled( _history.size() > 1 );
        _docPopupPrintBtn.setEnabled( _history.size() > 0 );

        // Display the popup menu
        _docPopupMenu.show( doc, x, y );
    }


    // ActionListener for document display events
    private class DocumentActionListener
    implements ActionListener
    {
        private String _doc;

        public DocumentActionListener( String doc )  { _doc = doc; }

        public void actionPerformed( ActionEvent e ) {
            actionDisplayDocument( pageToURL(_doc), true, true );
        }
    }

    public void actionTreePopup( int x, int y )
    {
        TreePath path = tree.getPathForLocation( x, y );
        if( path == null ) return;

        FamilyTreeNode node = (FamilyTreeNode) path.getLastPathComponent();
        if( node == null ) return;

        // Build the tree popup menu
        JPopupMenu treePopupMenu = new JPopupMenu( res.getString("FamilyTree") );
        JMenuItem treePopupDescBtn = new JMenuItem( res.getString("Description") );
        treePopupMenu.add( treePopupDescBtn );
        treePopupDescBtn.setEnabled( node.person.doc != null );
        treePopupDescBtn.addActionListener( new DocumentActionListener( node.person.doc ) );

        if( node.person.getBornPage() != null ) {
            JMenuItem treePopupBirthBtn = new JMenuItem( res.getString("Birth") );
            treePopupMenu.add( treePopupBirthBtn );
            treePopupBirthBtn.addActionListener( new DocumentActionListener( node.person.getBornPage() ) );
        }

        if( node.person.getChristenPage() != null ) {
            JMenuItem treePopupChristenBtn = new JMenuItem( res.getString("Christening") );
            treePopupMenu.add( treePopupChristenBtn );
            treePopupChristenBtn.addActionListener( new DocumentActionListener( node.person.getChristenPage() ) );
        }

        if( node.person.getDiedPage() != null ) {
            JMenuItem treePopupDeathBtn = new JMenuItem( res.getString("Death") );
            treePopupMenu.add( treePopupDeathBtn );
            treePopupDeathBtn.addActionListener( new DocumentActionListener( node.person.getDiedPage() ) );
        }

        if( node.person.getMarriedPage() != null ) {
            JMenuItem treePopupMarriageBtn = new JMenuItem( res.getString("Marriage") );
            treePopupMenu.add( treePopupMarriageBtn );
            treePopupMarriageBtn.addActionListener( new DocumentActionListener( node.person.getMarriedPage() ) );
        }

        // Display the popup menu
        treePopupMenu.show( tree, x, y );
    }

    public void setVisible(boolean b)
    {
        if (b)
            setLocation(50, 50);
        super.setVisible(b);
    }

    static public void main(String args[])
    {
        //UIManager.LookAndFeelInfo[] info = UIManager.getInstalledLookAndFeels();
        //for( int n = 0; n < info.length; n++ )
        //    System.out.println( info[n].getClassName() );

        JWindow splash = new SplashWindow();
        splash.setVisible(true);

        try {
            UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
        } catch( Exception e ) {
            System.out.println( e.getMessage() );
        }

        try {
            MainFrame frame = new MainFrame();

            if( args.length > 0 )
                frame.setFilename( args[0] );

            try { Thread.sleep( 2000 ); } catch( InterruptedException e ) {}

            frame.setVisible(true);
            new Thread( frame ).start();
        } catch( Exception e ) {
            System.out.println( e.getMessage() );
            /**/e.printStackTrace();
        }

        splash.setVisible(false);
    }

    public void addNotify()
    {
        // Record the size of the window prior to calling parents addNotify.
        Dimension size = getSize();

        super.addNotify();

        if (frameSizeAdjusted)
            return;
        frameSizeAdjusted = true;

        // Adjust size of frame according to the insets and menu bar
        Insets insets = getInsets();
        javax.swing.JMenuBar menuBar = getRootPane().getJMenuBar();
        int menuBarHeight = 0;
        if (menuBar != null)
            menuBarHeight = menuBar.getPreferredSize().height;
        setSize(insets.left + insets.right + size.width, insets.top + insets.bottom + size.height + menuBarHeight);
    }

    // Used by addNotify
    boolean frameSizeAdjusted = false;

    //{{DECLARE_CONTROLS
    javax.swing.JSplitPane horizsplit = new javax.swing.JSplitPane();
    javax.swing.JScrollPane treescrollpane = new javax.swing.JScrollPane();
    javax.swing.JTree tree = new javax.swing.JTree();
    javax.swing.JSplitPane vertsplit = new javax.swing.JSplitPane();
    javax.swing.JScrollPane tablescrollpane = new javax.swing.JScrollPane();
    javax.swing.JTable table = new javax.swing.JTable();
    javax.swing.JScrollPane docscrollpane = new javax.swing.JScrollPane();
    javax.swing.JEditorPane doc = new javax.swing.JEditorPane();
    javax.swing.JTextField statusTxt = new javax.swing.JTextField();
    javax.swing.JMenuBar menubar = new javax.swing.JMenuBar();
    javax.swing.JMenu fileMenu = new javax.swing.JMenu();
    javax.swing.JMenuItem fileOpenBtn = new javax.swing.JMenuItem();
    javax.swing.JMenuItem filePageSetupBtn = new javax.swing.JMenuItem();
    javax.swing.JMenuItem fileExitBtn = new javax.swing.JMenuItem();
    javax.swing.JMenu viewMenu = new javax.swing.JMenu();
    javax.swing.JMenu showMenu = new javax.swing.JMenu();
    javax.swing.JCheckBoxMenuItem showParentsChk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JCheckBoxMenuItem showChildrenChk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JCheckBoxMenuItem showSiblingsChk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JCheckBoxMenuItem showAuntsUnclesChk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JCheckBoxMenuItem showSpousesChk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JCheckBoxMenuItem showCousinsChk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JMenu orientMenu = new javax.swing.JMenu();
    javax.swing.JRadioButtonMenuItem orientNormalRb = new javax.swing.JRadioButtonMenuItem();
    javax.swing.JRadioButtonMenuItem orientInvertedRb = new javax.swing.JRadioButtonMenuItem();
    javax.swing.JMenu namingMenu = new javax.swing.JMenu();
    javax.swing.JRadioButtonMenuItem namingSurnameLastRb = new javax.swing.JRadioButtonMenuItem();
    javax.swing.JRadioButtonMenuItem namingSurnameFirstRb = new javax.swing.JRadioButtonMenuItem();
    javax.swing.JRadioButtonMenuItem namingSimpleNamesRb = new javax.swing.JRadioButtonMenuItem();
    javax.swing.JMenu viewSetRootMenu = new javax.swing.JMenu();
    javax.swing.JMenu toolsMenu = new javax.swing.JMenu();
    // new in 1.1.3
    javax.swing.JMenu toolsOptionsMenu = new javax.swing.JMenu();
    javax.swing.JCheckBoxMenuItem toolsOptionsPrune1Chk = new javax.swing.JCheckBoxMenuItem();
    javax.swing.JCheckBoxMenuItem toolsOptionsPrune2Chk = new javax.swing.JCheckBoxMenuItem();
    //
    javax.swing.JMenu helpMenu = new javax.swing.JMenu();
    javax.swing.JMenuItem helpAboutBtn = new javax.swing.JMenuItem();
    //}}
    javax.swing.ButtonGroup viewSetRootGrp = new ButtonGroup();
    javax.swing.JRadioButtonMenuItem[] viewSetRootMenuItems;
    // new in 1.1.3-rc5
    MyAction drawTreeAction = new MyAction("Draw_Tree", "drawtree.gif", 'T') {
        public void actionPerformed(ActionEvent actionevent) {
            actionDrawTree();
        }
    };
    MyAction drawAncestorsAction = new MyAction("Draw_Ancestors", "drawancest.gif", 'A') {
        public void actionPerformed(ActionEvent actionevent) {
            actionDrawAncestors();
        }
    };
    MyAction drawTimelineAction = new MyAction("Draw_Timeline", "drawtimeln.gif", 'L') {
        public void actionPerformed(ActionEvent actionevent) {
            actionDrawTimeline();
        }
    };
    javax.swing.JToolBar toolbar = new javax.swing.JToolBar("Toolbar"); 

    // Doc Popup Menu
    private JPopupMenu _docPopupMenu = new JPopupMenu( res.getString("FamilyTree") );
    private JMenuItem _docPopupBackBtn  = new JMenuItem( res.getString("Back") );
    private JMenuItem _docPopupPrintBtn = new JMenuItem( res.getString("Print_") );

    //{{DECLARE_MENUS
    //}}

    class SymAction implements java.awt.event.ActionListener
    {
        public void actionPerformed(java.awt.event.ActionEvent event)
        {
            Object object = event.getSource();
            if (object == fileExitBtn)
                fileExitBtn_actionPerformed(event);
        }
    }

    void fileExitBtn_actionPerformed(java.awt.event.ActionEvent event)
    {
        System.exit(0);
    }


    private void actionShowChanged()
    {
        int flags = 0;

        if( showParentsChk.isSelected() )       flags |= SHOW_PARENTS;
        if( showSpousesChk.isSelected() )       flags |= SHOW_SPOUSES;
        if( showChildrenChk.isSelected() )      flags |= SHOW_CHILDREN;
        if( showSiblingsChk.isSelected() )      flags |= SHOW_SIBLINGS;
        if( showAuntsUnclesChk.isSelected() )   flags |= SHOW_AUNTSUNCLES;
        if( showCousinsChk.isSelected() )       flags |= SHOW_COUSINS;

        _showFlags = flags;

        if( _lastSelectedNode != null )
            actionTreeNodeSelected( _lastSelectedNode );
    }


    private void updateScreenState()
    {
        // show menu state
        showParentsChk    .setSelected( isShowing(SHOW_PARENTS) );
        showSpousesChk    .setSelected( isShowing(SHOW_SPOUSES) );
        showChildrenChk   .setSelected( isShowing(SHOW_CHILDREN) );
        showSiblingsChk   .setSelected( isShowing(SHOW_SIBLINGS) );
        showAuntsUnclesChk.setSelected( isShowing(SHOW_AUNTSUNCLES) );
        showCousinsChk    .setSelected( isShowing(SHOW_COUSINS) );

        // naming menu state
        namingSurnameLastRb .setSelected( _namemode == MainFrame.NAMING_SURNAME_LAST );
        namingSurnameFirstRb.setSelected( _namemode == MainFrame.NAMING_SURNAME_FIRST );
        namingSimpleNamesRb .setSelected( _namemode == MainFrame.NAMING_SIMPLE_NAMES );

        // orient menu state
        orientNormalRb  .setSelected( _orientation == FamilyTreeModel.CONVENTIONAL );
        orientInvertedRb.setSelected( _orientation == FamilyTreeModel.INVERTED );

        // tools->options menu state
        toolsOptionsPrune1Chk.setSelected( _prune1 );
        toolsOptionsPrune2Chk.setSelected( _prune2 );

        // set root menu status
        for( int n = 0; n < viewSetRootMenuItems.length; n++ ) {
            if( _orientation == FamilyTreeModel.CONVENTIONAL ) {
                int m = ((_prune1 || _prune2) && (_prunedRootsCon != null)
                        && (n < _prunedRootsCon.length))  ? _prunedRootsCon[n] : n;
                        viewSetRootMenuItems[n].setSelected( _selectedConRoot == m );
            } else {
                int m = ((_prune1 || _prune2) && (_prunedRootsInv != null)
                        && (n < _prunedRootsInv.length)) ? _prunedRootsInv[n] : n;
                        viewSetRootMenuItems[n].setSelected( _selectedInvRoot == m );
            }
        }

        boolean menusEnabled = ( _people != null );

        filePageSetupBtn.setEnabled( menusEnabled );
        viewMenu.setEnabled( menusEnabled );
        toolsMenu.setEnabled( menusEnabled );

        // set title bar
        this.setTitle( "Taurus FamilyTree - " + _filename );
    }

    
    private void actionNamingChanged()
    {
        if( namingSurnameLastRb.isSelected() ) {
            _namemode = NAMING_SURNAME_LAST;
        } else if( namingSurnameFirstRb.isSelected() ) {
            _namemode = NAMING_SURNAME_FIRST;
        } else {
            _namemode = NAMING_SIMPLE_NAMES;
        }

    }

    private void actionOrientChanged()
    {
        if( orientInvertedRb.isSelected() ) {
            _orientation = FamilyTreeModel.INVERTED;
            buildSetRootMenu();
            //_root = _people[_selectedInvRoot];
        } else {
            _orientation = FamilyTreeModel.CONVENTIONAL;
            buildSetRootMenu();
            //_root = _people[_selectedConRoot];
        }

        //tree.setModel( _model = new FamilyTreeModel( _people, _root, _orientation ) );
        tree.setModel( _model = new FamilyTreeModel( _people, (_orientation == FamilyTreeModel.CONVENTIONAL)
                ? _prunedRootsCon : _prunedRootsInv, _orientation ) );
    }


    private boolean isShowing( int mask )
    {
        return ((_showFlags & mask) == mask);
    }


    /**
     * User has selected Tools->Options->Prune1.
     * Toggle the pruning of possible roots.
     */
    private void actionOptionsPrune1()
    {
        _prune1 = toolsOptionsPrune1Chk.isSelected();

        filterRoots();

        buildSetRootMenu();
    }


    /**
     * User has selected Tools->Options->Prune2.
     * Toggle the pruning of possible roots.
     */
    private void actionOptionsPrune2()
    {
        _prune2 = toolsOptionsPrune2Chk.isSelected();

        filterRoots();

        buildSetRootMenu();
    }


    private void actionDrawTree()
    {
        try {
            JFrame frame = new FTDiagramFrame( _people[_selectedConRoot], _namemode );
            frame.setVisible( true );
        } catch( Exception e ) {
            e.printStackTrace();
            System.out.println( e.toString() );
        }
    }


    private void actionDrawAncestors()
    {
        try {
            JFrame frame = new AncestorDiagramFrame( _people[_selectedInvRoot], _namemode );
            frame.setVisible( true );
        } catch( Exception e ) {
            e.printStackTrace();
            System.out.println( e.toString() );
        }
    }


    private void actionDrawTimeline()
    {
        try
        {
            JFrame frame = new TLDiagramFrame( _people );
            frame.setVisible( true );
        }
        catch( Exception e )
        {
            e.printStackTrace();
            System.out.println( e.toString() );
        }
    }


    /**
     * Filter people to produce prunedRootsConv and prunedRootsInv.
     */
    private void filterRoots()
    {
        int con = 0, inv = 0;
        int[] prunedRootsCon = new int[_people.length];
        int[] prunedRootsInv = new int[_people.length];

        for( int n = 0; n < _people.length; n++ )
        {
            boolean hasParent = (_people[n].father != null) || (_people[n].mother != null );
            boolean hasChildren = (_people[n].children != null) && !_people[n].children.isEmpty();

            // Conventional (Descendant) roots:
            // 1) no parents (top of tree)
            // 2) at least one child (tree shows something)
            if( (_prune1 && !_prune2 && !hasParent)
                    || (_prune2 && !_prune1 && hasChildren)
                    || (_prune1 && _prune2 && !hasParent && hasChildren) )
            {
                prunedRootsCon[con++] = n;
            }

            // Inverted (Ancestor) roots:
            // 1) no children (bottom of tree)
            // 2) at least one parent (tree shows something)
            if( (_prune1 && !_prune2 && !hasChildren)
                    || (_prune2 && !_prune1 && hasParent)
                    || (_prune1 && _prune2 && hasParent && !hasChildren) )
            {
                prunedRootsInv[inv++] = n;
            }
        }

        _prunedRootsCon = new int[con];
        System.arraycopy( prunedRootsCon, 0, _prunedRootsCon, 0, con );

        _prunedRootsInv = new int[inv];
        System.arraycopy( prunedRootsInv, 0, _prunedRootsInv, 0, inv );

        // Have we pruned?
        if( _prune1 || _prune2 )
        {
            // If root is 0, reset to first entry on menu
            if( (_selectedConRoot == 0) && (_prunedRootsCon.length > 0) )
                _selectedConRoot = _prunedRootsCon[0];

            if( (_selectedInvRoot == 0) && (_prunedRootsInv.length > 0) )
                _selectedInvRoot = _prunedRootsInv[0];
        }
    }


    /**
     * Build up the View->Set Root menu.  Given a set of roots, we go through
     * add their names to the menu, optionally pruning them as follows:
     *
     *   Normal   : Remove roots without any children
     *   Inverted : Remove roots without any parents
     *
     * Also split the menu into sections if it is too big for the screen, the
     * sections are linked by More...-> menus at the bottom.
     */
    private void buildSetRootMenu()
    {
        viewSetRootMenu.removeAll();
        viewSetRootGrp = new ButtonGroup();

        // If no roots, display <None> and return.
        if( (_people == null) || (_people.length == 0) )
        {
            viewSetRootMenuItems = new JRadioButtonMenuItem[1];
            viewSetRootMenuItems[0] = new JRadioButtonMenuItem( "<None>" );
            viewSetRootMenu.add( viewSetRootMenuItems[0] );
            updateScreenState();
            return;
        }

        if( _people.length == 1 )
        {
            viewSetRootMenuItems = new JRadioButtonMenuItem[1];

            // Create a menu option containin the details of the root
            StringBuffer label = new StringBuffer();
            if( _people[0].getSurname() != null )
                label.append( _people[0].getSurname().toUpperCase() );
            if( (_people[0].getSurname() != null) && (_people[0].getForenames() != null) )
                label.append( " " );
            if( _people[0].getForenames() != null )
                label.append( _people[0].getForenames() );

            viewSetRootMenuItems[0] = new JRadioButtonMenuItem( label.toString() );
            viewSetRootMenu.add( viewSetRootMenuItems[0] );
            updateScreenState();
            return;
        }

        // Get screen height
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int remain = screenSize.height - 64;

        // Simple class which remembers which menu it is on by means of an
        // offset into the list of (pruned) roots.
        class SetRootActionListener implements ActionListener {
            private int _offset;
            public SetRootActionListener( int offset ) {
                _offset = offset;
            }
            public void actionPerformed( final ActionEvent event ) {
                actionRootChanged( _offset );
            }
        }

        // ActionListener for the first set of menu items
        ActionListener setRootAction = new SetRootActionListener( 0 );

        // Inital menu
        JMenu setRootMenu = viewSetRootMenu;

        // Start of with largest number of people possible
        int max = _people.length;

        // Now use pruned size, if pruning in operation
        if( _prune1 || _prune2 )
            max = (_orientation == FamilyTreeModel.CONVENTIONAL)
            ? _prunedRootsCon.length : _prunedRootsInv.length;

        // Can now accurately create array of the correct size...
        viewSetRootMenuItems = new JRadioButtonMenuItem[max];

        // ...and fill it in
        for( int n = 0; n < max; n++ )
        {
            int m = n;

            // If we have pruned, translate the index to that of a person
            if( _prune1 || _prune2 )
            {
                m = (_orientation == FamilyTreeModel.CONVENTIONAL)
                ? _prunedRootsCon[n] : _prunedRootsInv[n];
            }	    

            // Create a menu option containin the details of the root
            StringBuffer label = new StringBuffer();
            if( _people[m].getSurname() != null )
                label.append( _people[m].getSurname().toUpperCase() );
            if( (_people[m].getSurname() != null) && (_people[m].getForenames() != null) )
                label.append( " " );
            if( _people[m].getForenames() != null )
                label.append( _people[m].getForenames() );

            viewSetRootMenuItems[n] = new JRadioButtonMenuItem( label.toString() );

            // Have we enough space left?
            if( remain < 0 )
            {
                // Create a sub-menu and reset the height counter
                JMenu moreMenu = new JMenu( "More..." );
                setRootAction = new SetRootActionListener( n );
                setRootMenu.add( moreMenu );
                setRootMenu = moreMenu;
                remain = screenSize.height - 64;
            }

            // Actually add the menu item to the menu
            setRootMenu.add( viewSetRootMenuItems[n] );
            viewSetRootGrp.add( viewSetRootMenuItems[n] );
            viewSetRootMenuItems[n].addActionListener( setRootAction );

            // Decrement the height counter
            remain -= 22;
        }

        updateScreenState();
    }


    /**
     * Called when the user selects a new root to be displayed.  The offset
     * indicates whether this came from a sub-menu, and if so, as what point
     * that menu started.
     */
    private void actionRootChanged( int offset )
    {
        for( int n = 0; n < viewSetRootMenuItems.length; n++ ) {
            if( viewSetRootMenuItems[n].isSelected() ) {
                // If menu has been pruned, translate index into roots index                    
                if( _orientation == FamilyTreeModel.CONVENTIONAL ) {
                    int m = (_prune1 || _prune2) ? _prunedRootsCon[n] : n;
                    _selectedConRoot = m;
                } else {
                    int m = (_prune1 || _prune2) ? _prunedRootsInv[n] : n;
                    _selectedInvRoot = m;
                }

                break;
            }
        }

        actionOrientChanged();
    }

    private void actionOpen()
    {
        JFileChooser opendlg = new JFileChooser( "." );
        javax.swing.filechooser.FileFilter geneFilter;
        opendlg.addChoosableFileFilter( geneFilter = new javax.swing.filechooser.FileFilter() {
            public boolean accept( java.io.File f ) {
                String name = f.getName().toLowerCase();
                return name.endsWith( ".ftml" ) || name.endsWith( ".xml" )
                    || name.endsWith( ".ged" ) || name.endsWith( ".csv" )
                    || f.isDirectory();
            }
            public String getDescription() { return "Genealogy files (*.ftml, *.xml, *.ged, *.csv)"; }
        });
        opendlg.addChoosableFileFilter( new javax.swing.filechooser.FileFilter() {
            public boolean accept( java.io.File f ) {
                String name = f.getName().toLowerCase();
                return name.endsWith( ".ftml" ) || name.endsWith( ".xml" )
                    || f.isDirectory();
            }
            public String getDescription() { return "FTML files (*.ftml, *.xml)"; }
        });
        opendlg.addChoosableFileFilter( new javax.swing.filechooser.FileFilter() {
            public boolean accept( java.io.File f ) {
                return f.getName().toLowerCase().endsWith( ".ged" ) || f.isDirectory();
            }
            public String getDescription() { return "Gedcom files (*.ged)"; }
        });
        opendlg.addChoosableFileFilter( new javax.swing.filechooser.FileFilter() {
            public boolean accept( java.io.File f ) {
                return f.getName().toLowerCase().endsWith( ".csv" ) || f.isDirectory();
            }
            public String getDescription() { return "CSV files (*.csv)"; }
        });
        opendlg.setFileFilter( geneFilter );

        if( opendlg.showOpenDialog( this ) == JFileChooser.APPROVE_OPTION )
        {
            java.io.File file = opendlg.getSelectedFile();
            setFilename( file.toString() );
            new Thread( this ).start();
        }
    }

    private void actionPageSetup()
    {
        new PageSetupDialog( this, false ).setVisible( true );
    }


    private void setBusy( String message )
    {
        if( message != null ) {
            statusTxt.setText( message );
            statusTxt.repaint();
            setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
        } else {
            setCursor( Cursor.getDefaultCursor() );
            statusTxt.setText( "" );
            statusTxt.repaint();
        }
    }
    
    
    private void createPageForPerson(Person person)
    {
        doc.setDocument( new HTMLDocument() );
        doc.setContentType("text/html");
        StringBuffer text = new StringBuffer();
        text.append("<b>Name:</b> " + person.getName() + "<br>");
        text.append("<b>Born:</b> " + person.getBorn() + "<br>");
        if(person.spouses != null) {
            text.append("<b>Married:</b> ");
            for(Person spouse : person.spouses) {
                text.append(spouse.getName());
            }
            text.append("<br>");
        }
        text.append("<b>Died:</b> " + person.getDied());
        doc.setText(text.toString());
    }


    private void reportException( Exception e )
    {
        StringWriter out = new StringWriter();
        PrintWriter pout = new PrintWriter( out );
        pout.println( e.toString() );
        e.printStackTrace( pout );
        JOptionPane.showMessageDialog( this, out.toString(), res.getString("AException_has"), JOptionPane.ERROR_MESSAGE );
        e.printStackTrace();
        System.out.println( e );
    }
}