///////////////////////////////////////////////////////////////////////////////
//
// Filename: TLDiagramFrame.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  Window for Time Line Diagram.
//
// (c) Copyright R.H.Thornburrow, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.timeline;

import com.tsac.familytree.*;
import com.tsac.familytree.gui.FamilyTreeGridBagConstraints;
import com.tsac.familytree.gui.PageSetupDialog;

import java.awt.GridBagLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.PrintJob;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Cursor;
import java.util.ResourceBundle;

@SuppressWarnings("serial")
public class TLDiagramFrame
    extends javax.swing.JFrame
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");

    public TLDiagramFrame( Person[] people )
    {
        //{{INIT_CONTROLS
        setJMenuBar(menubar);
        getContentPane().setLayout(new GridBagLayout());
        setSize(800,600);
        setVisible(false);
        getContentPane().add(diagscroll,new FamilyTreeGridBagConstraints(0,0,1,1,1.0,1.0,java.awt.GridBagConstraints.CENTER,java.awt.GridBagConstraints.BOTH,new Insets(0,0,0,0),0,0));
        diagscroll.getViewport().setBackground(java.awt.Color.white);
        diagscroll.setBounds(0,0,20,40);
        diagram.setLayout(null);
        diagscroll.getViewport().add(diagram);
        diagram.setBackground(java.awt.Color.white);
        diagram.setBounds(0,0,430,270);
        //$$ menubar.move(0,601);
        fileMenu.setActionCommand("File");
        menubar.add(fileMenu);
        filePageSetupBtn.setActionCommand("Page Setup...");
        fileMenu.add(filePageSetupBtn);
        filePrintBtn.setActionCommand("Print...");
        fileMenu.add(filePrintBtn);
        fileCloseBtn.setActionCommand("Close");
        fileMenu.add(fileCloseBtn);
        //$$ JMenuBar1.move(0,601);
        setTitle("A Simple Frame");
        //}}

        //{{REGISTER_LISTENERS
        SymWindow aSymWindow = new SymWindow();
        this.addWindowListener(aSymWindow);
        //}}

        //{{INIT_MENUS
        //}}

        fileMenu.setText(res.getString("File"));
        filePageSetupBtn.setText(res.getString("Page_Setup_"));
        filePrintBtn.setText(res.getString("Print_"));
        fileCloseBtn.setText(res.getString("Close"));

        filePageSetupBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                actionPageSetup();
            }
        });

        filePrintBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                actionPrint();
            }
        });

        fileCloseBtn.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                actionClose();
            }
        });

        diagram.setData( people );
        this.setTitle( res.getString("Timeline") );
    }


    public void setVisible(boolean b)
    {
        if(b)
        {
            setLocation(50, 50);
        }
        super.setVisible(b);
    }


    public void addNotify()
    {
        // Record the size of the window prior to calling parents addNotify.
        Dimension d = getSize();

        super.addNotify();

        if (fComponentsAdjusted)
            return;

        // Adjust components according to the insets
        Insets ins = getInsets();
        setSize(ins.left + ins.right + d.width, ins.top + ins.bottom + d.height);
        Component components[] = getContentPane().getComponents();
        for (int i = 0; i < components.length; i++)
        {
            Point p = components[i].getLocation();
            p.translate(ins.left, ins.top);
            components[i].setLocation(p);
        }
        fComponentsAdjusted = true;
    }

    // Used for addNotify check.
    boolean fComponentsAdjusted = false;

    class SymWindow extends java.awt.event.WindowAdapter
    {
        public void windowClosing(java.awt.event.WindowEvent event)
        {
            Object object = event.getSource();
            if (object == TLDiagramFrame.this)
                TLDiagramFrame_WindowClosing(event);
        }
    }

    void TLDiagramFrame_WindowClosing(java.awt.event.WindowEvent event)
    {
        dispose();		 // dispose of the Frame.
    }
    //{{DECLARE_CONTROLS
    javax.swing.JScrollPane diagscroll = new javax.swing.JScrollPane();
    com.tsac.familytree.timeline.TLDiagram diagram = new com.tsac.familytree.timeline.TLDiagram();
    javax.swing.JMenuBar menubar = new javax.swing.JMenuBar();
    javax.swing.JMenu fileMenu = new javax.swing.JMenu();
    javax.swing.JMenuItem filePageSetupBtn = new javax.swing.JMenuItem();
    javax.swing.JMenuItem filePrintBtn = new javax.swing.JMenuItem();
    javax.swing.JMenuItem fileCloseBtn = new javax.swing.JMenuItem();
    //}}

    //{{DECLARE_MENUS
    //}}


    private void actionPageSetup()
    {
        new PageSetupDialog( this, false ).setVisible( true );
    }


    private void actionPrint()
    {
        PrintJob job = Toolkit.getDefaultToolkit().getPrintJob( this, res.getString("Timeline"), null );

        if( job == null ) return;

        setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
        Graphics jg = job.getGraphics();
        Dimension psz = job.getPageDimension();

        try
        {
            Graphics pg = jg.create(
                    PageSetupDialog.leftMargin, PageSetupDialog.topMargin,
                    psz.width - (PageSetupDialog.leftMargin + PageSetupDialog.rightMargin),
                    psz.height - (PageSetupDialog.topMargin + PageSetupDialog.bottomMargin) );
            Dimension size = new Dimension(
                    psz.width - (PageSetupDialog.leftMargin + PageSetupDialog.rightMargin),
                    psz.height - (PageSetupDialog.topMargin + PageSetupDialog.bottomMargin) );
            diagram.print( pg, size );
            pg.dispose();
        }
        catch( Exception e )
        {
            e.printStackTrace();
            System.out.println( e );
        }
        finally
        {
            jg.dispose();
            job.end();
            job = null;
            setCursor( Cursor.getDefaultCursor() );
        }
    }

    private void actionClose()
    {
        this.setVisible( false );
    }
}