///////////////////////////////////////////////////////////////////////////////
//
// Filename: ApproxDate.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001-4
//
// Purpose:  Representation of an approximate date (mm/yyyy or yyyy or Before,
//           After or About and any form of date).
//
// (c) Copyright R.H.Thornburrow, 2001-4
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.util;

import java.util.ResourceBundle;


public class ApproxDate
    extends DateBase
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");

    // new in 1.1.3
    public static final int NONE   = 0;
    public static final int BEFORE = 1;
    public static final int AFTER  = 2;
    public static final int ABOUT  = 3;

    public static final int DAY_UNKNOWN   = -1;
    public static final int MONTH_UNKNOWN = -1;

    // new in 1.1.3
    protected int _befAftAbt;
    
    private int _day;
    private int _month;
    private int _year;
    
    public ApproxDate( int year )
    {
        _day   = DAY_UNKNOWN;
        _month = MONTH_UNKNOWN;
        _year  = year;
    }
    
    // new in 1.1.3
    public ApproxDate( int month, int year )
    {
        _day   = DAY_UNKNOWN;
        _month = month;
        _year  = year;
    }
    
    // new in 1.1.3
    public ApproxDate( int day, int month, int year )
    {
        _day   = day;        
        _month = month;
        _year  = year;
    }


    // new in 1.1.3
    public void setBeforeAfterAbout( int befAftAbt )
    {
        _befAftAbt = befAftAbt;
    }

    // new in 1.1.3
    public int getBeforeAfterAbout()
    {
        return _befAftAbt;
    }
    
    
    /**
     * Format the string from the available data - can produce:
     *
     *   2004
     *   06/2004
     *   19/06/2004 (should be an AbsoluteDate)
     *   Before 2004
     *   After 2004
     *   About 2004
     *   Before 06/2004
     *   After 06/2004
     *   About 06/2004
     *   Before 19/06/2004
     *   After 19/06/2004
     *   About 19/06/2004
     */
    public String toString()
    {
        StringBuffer ret = new StringBuffer();
        
        switch( _befAftAbt )
        {
        case BEFORE: ret.append( res.getString("Before") ).append( " " ); break;
        case AFTER:  ret.append( res.getString("After") ).append( " " );  break;
        case ABOUT:  ret.append( res.getString("About") ).append( " " );  break;
        }

        if( (_day != DAY_UNKNOWN) && (_month != MONTH_UNKNOWN) )
        {
            String daystr = Integer.toString( _day );
            if( daystr.length() < 2 ) ret.append( "0" );
            ret.append( daystr );
            ret.append( "/" );
        }
        
        if( _month != MONTH_UNKNOWN )
        {
            String mthstr = Integer.toString( _month );
            if( mthstr.length() < 2 ) ret.append( "0" );
            ret.append( mthstr );
            ret.append( "/" );
        }
        
        ret.append( Integer.toString( _year ) );
        return ret.toString();
    }


    /**
     * Returns the month as a integer in the range 0-11.  Return -1 if month is
     * not set.  This is intended for use in calculations and matches the range
     * returned by AbsoluteDate.
     */
    public int getMonth()
    {
        if( _month != -1 )
            return _month - 1;
            
        return -1;
    }
    
    
    /**
     * Returns the year as a simple integer.
     */
    public int getYear()
    {
        return _year;
    }
    
    
    /**
     * Returns the year as an abbreviated string, where "<" mean "Before", ">"
     * means "After" and "~" means "About".  If there is no symbol, then the
     * year is exact.  This is used in displaying trees, where a concise date
     * format is needed.  These are universal symbols, so no need for
     * translation.
     */
    public String getYearString()
    {
        StringBuffer ret = new StringBuffer();
        
        switch( _befAftAbt )
        {
        case BEFORE: ret.append( "<" ); break;
        case AFTER:  ret.append( ">" );  break;
        case ABOUT:  ret.append( "~" );  break;
        }
        
        ret.append( Integer.toString( _year ) );
        return ret.toString();
    }
}
