///////////////////////////////////////////////////////////////////////////////
//
// Filename: DateRange.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  26th June 2004
//
// Purpose:  Representation of an approximate date somewhere between two give
//           dates (new in 1.1.3)
//
// (c) Copyright R.H.Thornburrow, 2004
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.util;

import java.util.ResourceBundle;


public class DateRange
    extends DateBase
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");

    private DateBase _from;
    private DateBase _to;
    
    
    public DateRange( DateBase from, DateBase to )
    {
        _from = from;
        _to   = to;
    }
    
    
    /**
     * Format the string from the available data - can produce:
     *
     *   Between 2003 and 2004
     *   Between 11/2003 and 12/2003
     */
    public String toString()
    {
        StringBuffer ret = new StringBuffer( res.getString("Between") );
        
        ret.append( " " );
        ret.append( _from );
        ret.append( " " );
        ret.append( res.getString("and") );
        ret.append( " " );
        ret.append( _to );
        
        return ret.toString();
    }


    /**
     * Returns the month as a integer in the range 0-11.  Returns -1 if the
     * month and year are not the same in both from and to dates.  This is
     * intended for use in calculations and matches the range returned by
     * ApproxDate and AbsoluteDate.
     */
    public int getMonth()
    {
        if( (_from.getYear() != -1)
            && (_from.getYear() == _to.getYear())
            && (_from.getMonth() != -1)
            && (_from.getMonth() == _to.getMonth()) )
            return _from.getMonth();
            
        return -1;
    }
    
    
    /**
     * Returns the year as a simple integer.  Returns -1 if the year is not the
     * same in both from and to dates.
     */
    public int getYear()
    {
        if( (_from.getYear() != -1)
            && (_from.getYear() == _to.getYear()) )
            return _from.getYear();
            
        return -1;
    }
    
    
    /**
     * Returns the year, where both from and to are in the same year, and a
     * string of the form xxxx-yyyy where the from and to years are different.
     */
    public String getYearString()
    {
        if( (_from.getYear() != -1)
            && (_from.getYear() == _to.getYear()) )
            return Integer.toString( _from.getYear() );

        StringBuffer ret = new StringBuffer();
        
        ret.append( Integer.toString( _from.getYear() ) );
        ret.append( "-" );        
        ret.append( Integer.toString( _to.getYear() ) );
        return ret.toString();
    }
}
