///////////////////////////////////////////////////////////////////////////////
//
// Filename: DateUtil.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  20th May 2001
//
// Purpose:  Date manipulation utilities.
//
// (c) Copyright R.H.Thornburrow, 2001-4
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.util;

import java.util.*;
import java.text.*;

import java.util.ResourceBundle;


public class DateUtil
{
    static ResourceBundle res = ResourceBundle.getBundle("FamilyTree");

    public static DateFormat ddmmyyyy   = new SimpleDateFormat( "dd/MM/yyyy" );
    public static DateFormat mmyyyy     = new SimpleDateFormat( "MM/yyyy" );     // new in 1.1.3
    public static DateFormat ddmmmyyyy  = new SimpleDateFormat( "dd MMM yyyy" );
    public static DateFormat mmmyyyy    = new SimpleDateFormat( "MMM yyyy" );    // new in 1.1.3
    public static DateFormat yyyy       = new SimpleDateFormat( "yyyy" );

    public static DateFormat[] formats = {
        ddmmyyyy, mmyyyy, ddmmmyyyy, mmmyyyy, yyyy
    };

    public static DateBase toDateBase( String str )
        throws DateException
    {
        return toDateBase( str, ApproxDate.NONE, 0 );
    }
    
    public static DateBase toDateBase( String str, int befAftAbt, int line )
        throws DateException
    {
        if( str == null )
            return null;

        DateBase ret = null;
        int n;
        
        for( /*int*/ n = 0; (ret == null) && (n < formats.length); n++ )
        {
            try
            {
                Date date;
                Calendar cal= Calendar.getInstance();
                
                switch( n )
                {
                case 0:     // 31/12/2001
                    date = ddmmyyyy.parse( str );
                    cal.setTime(date);
                    if( befAftAbt == ApproxDate.NONE ) {
                        ret = new AbsoluteDate( cal );
                    } else {
                        ret = new ApproxDate( cal.get(Calendar.DATE),
                                              cal.get(Calendar.MONTH)+1,
                                              cal.get(Calendar.YEAR) );
                        ((ApproxDate) ret).setBeforeAfterAbout( befAftAbt );
                    }
                    break;
                case 1:     // 12/2001
                    date = mmyyyy.parse( str );
                    cal.setTime(date);
                    ret = new ApproxDate( cal.get(Calendar.MONTH)+1, cal.get(Calendar.YEAR) );
                    ((ApproxDate) ret).setBeforeAfterAbout( befAftAbt );
                    break;
                case 2:     // 31 Dec 2001 or 31 December 2001
                    date = ddmmmyyyy.parse( str );
                    cal.setTime(date);
                    if( befAftAbt == ApproxDate.NONE ) {
                        ret = new AbsoluteDate( cal );
                    } else {
                        ret = new ApproxDate( cal.get(Calendar.DATE),
                                              cal.get(Calendar.MONTH)+1,
                                              cal.get(Calendar.YEAR) );
                        ((ApproxDate) ret).setBeforeAfterAbout( befAftAbt );
                    }
                    break;
                case 3:     // Dec 2001 or December 2001
                    date = mmmyyyy.parse( str );
                    cal.setTime(date);
                    ret = new ApproxDate( cal.get(Calendar.MONTH)+1, cal.get(Calendar.YEAR) );
                    ((ApproxDate) ret).setBeforeAfterAbout( befAftAbt );
                    break;
                case 4:     // 2001
                    if( str.matches( "[0-9]+" ) ) // yyyy doesn't give parse error if not all digits
                    {
                        date = yyyy.parse( str );
                        cal.setTime(date);
                        ret = new ApproxDate( cal.get(Calendar.YEAR) );
                        ((ApproxDate) ret).setBeforeAfterAbout( befAftAbt );
                    }
                    break;
                }
            }
            catch( Exception e )
            {
            }
        }
        
        if( ret == null ) {
            if( line != 0 ) {
                throw new DateException("PARSE ERROR: " + line + ": Cannot parse date \"" + str + "\"");
            } else {
                throw new DateException( "PARSE ERROR: Cannot parse date \"" + str + "\"" );
            }            
        }
        
        return ret;
    }
    
    public static String toString( Calendar cal )
    {
        if( cal != null )
            return ddmmyyyy.format( cal.getTime() );
            
        return " ";
    }
    
    public static int difference( Calendar start, Calendar end )
    {
        int years = end.get( Calendar.YEAR ) - start.get( Calendar.YEAR );
        int days = end.get( Calendar.DAY_OF_YEAR ) - start.get( Calendar.DAY_OF_YEAR );
        
        if( days >= 0 ) return years;
        
        return (years - 1);
    }
    
    public static String differenceString( Calendar start, Calendar end )
    {
        long startdays = start.get( Calendar.YEAR )*365 + start.get( Calendar.DAY_OF_YEAR );
        long enddays   = end.get( Calendar.YEAR )*365   + end.get( Calendar.DAY_OF_YEAR );
        long diff = enddays - startdays;

        // Anything less than 2 weeks is reported in days
        if( diff < 2*7 )
            return diff + " " + res.getString("days");
        
        // Anything less than 2 months is reported in weeks
        if( diff < 2*31 )
            return (diff / 7) + " " + res.getString("weeks");
        
        // Anything less than 2 years is reported in months
        if( diff < 2*365 )
            return ((diff*12) / 365) + " " + res.getString("months");

        // Otherwise just do it whole years
        return Long.toString( diff / 365 );
    }
}