///////////////////////////////////////////////////////////////////////////////
//
// Filename: GedcomXmlHandler.java
//
// Author:   Robert Thornburrow
// Project:  FamilyTree
// Created:  27th February 2010
//
// Purpose:  Gedcom XML SAX Handler for use with Xerces.
//
// (c) Copyright R.H.Thornburrow, 2001-10
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
//
///////////////////////////////////////////////////////////////////////////////

package com.tsac.familytree.xml;

import com.tsac.familytree.*;
import com.tsac.familytree.util.LogBuffer;
import com.tsac.familytree.xml.ged2x.Ged2XXmlHandler;
import com.tsac.familytree.xml.gedcom.GedcomXmlHandler;
import com.tsac.ftml.parser.FtmlHandler;

import org.xml.sax.SAXException;
import org.xml.sax.Attributes;

import java.util.Vector;

/**
 * Sneak class to figure out what kind of XML file we have been given and
 * instantiate a handler for it based on the first opening tag.  All subsequent
 * calls are then delegated to the real handler.
 * 
 * @author Robert
 */
public class GenericXmlHandler
    extends org.xml.sax.helpers.DefaultHandler
{
    private IXmlHandlerDelegate _delegate = null;
    private LogBuffer _log;

    /**
     * Constructor.
     */
    public GenericXmlHandler(LogBuffer log)
    {
        _log = log;
    }


    public Vector<Person> getPeople()
    {
        if(_delegate != null)
            return _delegate.getPeople();
        return null;
    }
    
    public Vector<Person> getRoots()
    {
        if(_delegate != null)
            return _delegate.getRoots();
        return null;
    }
    
    public Vector<Marriage> getMarriages()
    {
        if(_delegate != null)
            return _delegate.getMarriages();
        return null;
    }


    @Override
    public void endDocument()
        throws SAXException
    {
        if(_delegate != null)
            _delegate.endDocument();
    }


    /**
     * Handles the start of each element.  Creates a new object for those that
     * need one, or sets an property for those that don't.
     */
    public void startElement( String uri, String localName, String qName, Attributes attributes )
        throws SAXException
    {
        if(_delegate == null) {
            if("gedcom".equalsIgnoreCase(qName)) {
                _delegate = new GedcomXmlHandler(_log);
            } else if("ftml".equalsIgnoreCase(qName)) {
                _delegate = new FtmlHandler(_log);
            } else if("family_tree".equalsIgnoreCase(qName)) {
                _delegate = new Ged2XXmlHandler(_log);
            } else {
                throw new SAXException("PARSE ERROR: Unrecognised XML format");
            }
        }
        if(_delegate != null) {
            _delegate.startElement(uri, localName, qName, attributes);
            return;
        }
    }
    
    
    /**
     * Handles the end of each element.  Saves any objects that need saving.
     */
    public void endElement( String uri, String localName, String qName )
        throws SAXException
    {
        if(_delegate != null) {
            _delegate.endElement(uri, localName, qName);
        }
    }
    
    public void characters(char[] ch, int start, int length)
        throws SAXException
    {
        if(_delegate != null) {
            _delegate.characters(ch, start, length);
        }
    }
}